/* 
 * Copyright (c) 2023 Emma Tebibyte
 * SPDX-License-Identifier: AGPL-3.0-or-later
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see https://www.gnu.org/licenses/.
 */

use std::{
    collections::HashMap,
    net::{ TcpListener, TcpStream },
    sync::{ Arc, Mutex },
    thread,
};

use liminality::handle_client;

fn main() {
    let listener = TcpListener::bind("0.0.0.0:8080")
        .expect("Failed to bind to port 8080.");
    println!("Server listening on port 8080.");

    let connections: Arc<Mutex<HashMap<usize, TcpStream>>> = Arc::new(
        Mutex::new(HashMap::new())
    );

    for (id, stream) in listener.incoming().enumerate() {
        match stream {
            Ok(stream) => {
                let clients = connections.clone();
                clients.lock().unwrap().insert(id.clone(), stream);
                thread::spawn(move || handle_client(id, clients));
            }
            Err(err) => {
                eprintln!("Error accepting client connection: {}", err);
            }
        }
    }
}
